@@ -54,6 +54,7 @@ enum
     STATUS_NEED_PASSPHRASE,
     STATUS_VALIDSIG,
     STATUS_ASSERT_SIGNER,
+    STATUS_ASSERT_PUBKEY_ALGO,
     STATUS_SIG_ID,
     STATUS_ENC_TO,
     STATUS_NODATA,
@@ -152,6 +153,7 @@ enum
     STATUS_TRUNCATED,
     STATUS_MOUNTPOINT,
     STATUS_BLOCKDEV,
+    STATUS_PLAINDEV,           /* The decrypted virtual device.  */
 
     STATUS_PINENTRY_LAUNCHED,
 
