@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef GPG_KEYRING_H
@@ -40,6 +40,6 @@ int keyring_delete_keyblock (KEYRING_HANDLE hd);
 int keyring_search_reset (KEYRING_HANDLE hd);
 int keyring_search (KEYRING_HANDLE hd, KEYDB_SEARCH_DESC *desc,
 		    size_t ndesc, size_t *descindex, int skip_legacy);
-int keyring_rebuild_cache (void *token,int noisy);
+int keyring_rebuild_cache (ctrl_t ctrl, void *token,int noisy);
 
 #endif /*GPG_KEYRING_H*/
