@@ -27,13 +27,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 #include <time.h>
 
 #include "scdaemon.h"
 
 #include "iso7816.h"
-#include "app-common.h"
 #include "../common/tlv.h"
 #include "apdu.h"
 
@@ -484,7 +482,8 @@ read_ef_prkd (app_t app, unsigned short fid, prkdf_object_t *prkdresult,
   if (!fid)
     return gpg_error (GPG_ERR_NO_DATA); /* No private keys. */
 
-  err = select_and_read_binary (app->slot, fid, "PrKDF", &buffer, &buflen, 255);
+  err = select_and_read_binary (app_get_slot (app),
+                                fid, "PrKDF", &buffer, &buflen, 255);
   if (err)
     return err;
 
@@ -667,6 +666,7 @@ read_ef_prkd (app_t app, unsigned short fid, prkdf_object_t *prkdresult,
       {
         /* Yep, this is the keyReference.
            Note: UL is currently not used. */
+        (void)ul;
         for (ul=0; objlen; objlen--)
           {
             ul <<= 8;
@@ -832,7 +832,7 @@ read_ef_prkd (app_t app, unsigned short fid, prkdf_object_t *prkdresult,
   xfree (buffer);
   buffer = NULL;
   buflen = 0;
-  err = select_and_read_binary (app->slot,
+  err = select_and_read_binary (app_get_slot (app),
                                 ((SC_HSM_EE_PREFIX << 8) | (fid & 0xFF)),
                                 "CertEF", &buffer, &buflen, 1);
   if (!err && buffer[0] == 0x30)
@@ -953,7 +953,8 @@ read_ef_cd (app_t app, unsigned short fid, cdf_object_t *result)
   if (!fid)
     return gpg_error (GPG_ERR_NO_DATA); /* No certificates. */
 
-  err = select_and_read_binary (app->slot, fid, "CDF", &buffer, &buflen, 255);
+  err = select_and_read_binary (app_get_slot (app), fid, "CDF",
+                                &buffer, &buflen, 255);
   if (err)
     return err;
 
@@ -1202,7 +1203,7 @@ read_serialno(app_t app)
   size_t n, objlen, hdrlen, chrlen;
   int class, tag, constructed, ndef;
 
-  err = select_and_read_binary (app->slot, 0x2F02, "EF.C_DevAut",
+  err = select_and_read_binary (app_get_slot (app), 0x2F02, "EF.C_DevAut",
                                 &buffer, &buflen, 512);
   if (err)
     return err;
@@ -1229,15 +1230,15 @@ read_serialno(app_t app)
     }
   chrlen -= 5;
 
-  app->serialno = xtrymalloc (chrlen);
-  if (!app->serialno)
+  app->card->serialno = xtrymalloc (chrlen);
+  if (!app->card->serialno)
     {
       err = gpg_error_from_syserror ();
       goto leave;
     }
 
-  app->serialnolen = chrlen;
-  memcpy (app->serialno, chr, chrlen);
+  app->card->serialnolen = chrlen;
+  memcpy (app->card->serialno, chr, chrlen);
 
  leave:
   xfree (buffer);
@@ -1260,7 +1261,7 @@ read_meta (app_t app)
   if (err)
     return err;
 
-  err = list_ef (app->slot, &eflist, &eflistlen);
+  err = list_ef (app_get_slot (app), &eflist, &eflistlen);
   if (err)
     return err;
 
@@ -1354,7 +1355,7 @@ keygripstr_from_prkdf (app_t app, prkdf_object_t prkdf, char *r_gripstr)
     err = ksba_cert_init_from_mem (cert, der, derlen);
   xfree (der);
   if (!err)
-    err = app_help_get_keygrip_string (cert, r_gripstr, NULL);
+    err = app_help_get_keygrip_string (cert, r_gripstr, NULL, NULL);
   ksba_cert_release (cert);
 
   return err;
@@ -1387,7 +1388,7 @@ send_keypairinfo (app_t app, ctrl_t ctrl, prkdf_object_t keyinfo)
         }
       else
         {
-          assert (strlen (gripstr) == 40);
+          log_assert (strlen (gripstr) == 40);
           send_status_info (ctrl, "KEYPAIRINFO",
                             gripstr, 40,
                             buf, strlen (buf),
@@ -1406,7 +1407,7 @@ do_learn_status (app_t app, ctrl_t ctrl, unsigned int flags)
 {
   gpg_error_t err;
 
-  if ((flags & 1))
+  if ((flags & APP_LEARN_FLAG_KEYPAIRINFO))
     err = 0;
   else
     {
@@ -1454,7 +1455,7 @@ readcert_by_cdf (app_t app, cdf_object_t cdf,
       return 0;
     }
 
-  err = select_and_read_binary (app->slot, cdf->fid, "CD",
+  err = select_and_read_binary (app_get_slot (app), cdf->fid, "CD",
                                 &buffer, &buflen, 4096);
   if (err)
     {
@@ -1483,7 +1484,7 @@ readcert_by_cdf (app_t app, cdf_object_t cdf,
       goto leave;
     }
   totobjlen = objlen + hdrlen;
-  assert (totobjlen <= buflen);
+  log_assert (totobjlen <= buflen);
 
   err = parse_ber_header (&p, &n, &class, &tag, &constructed,
                           &ndef, &objlen, &hdrlen);
@@ -1514,7 +1515,7 @@ readcert_by_cdf (app_t app, cdf_object_t cdf,
           goto leave;
         }
       totobjlen = objlen + hdrlen;
-      assert (save_p + totobjlen <= buffer + buflen);
+      log_assert (save_p + totobjlen <= buffer + buflen);
       memmove (buffer, save_p, totobjlen);
     }
 
@@ -1592,7 +1593,8 @@ do_getattr (app_t app, ctrl_t ctrl, const char *name)
     }
   else if (!strcmp (name, "$DISPSERIALNO"))
     {
-      send_status_info (ctrl, name, app->serialno, app->serialnolen, NULL, 0);
+      send_status_info (ctrl, name,
+                        app->card->serialno, app->card->serialnolen, NULL, 0);
       return 0;
     }
 
@@ -1693,8 +1695,8 @@ verify_pin (app_t app, gpg_error_t (*pincb)(void*, const char *, char **),
   char *prompt;
   int sw;
 
-  sw = apdu_send_simple (app->slot, 0, 0x00, ISO7816_VERIFY, 0x00, 0x81,
-                         -1, NULL);
+  sw = apdu_send_simple (app_get_slot (app),
+                         0, 0x00, ISO7816_VERIFY, 0x00, 0x81, -1, NULL);
 
   if (sw == SW_SUCCESS)
     return 0;                   /* PIN already verified */
@@ -1719,7 +1721,7 @@ verify_pin (app_t app, gpg_error_t (*pincb)(void*, const char *, char **),
   prompt = "||Please enter the PIN";
 
   if (!opt.disable_pinpad
-      && !iso7816_check_pinpad (app->slot, ISO7816_VERIFY, &pininfo) )
+      && !iso7816_check_pinpad (app_get_slot (app), ISO7816_VERIFY, &pininfo) )
     {
       err = pincb (pincb_arg, prompt, NULL);
       if (err)
@@ -1728,7 +1730,7 @@ verify_pin (app_t app, gpg_error_t (*pincb)(void*, const char *, char **),
           return err;
         }
 
-      err = iso7816_verify_kp (app->slot, 0x81, &pininfo);
+      err = iso7816_verify_kp (app_get_slot (app), 0x81, &pininfo);
       pincb (pincb_arg, NULL, NULL);  /* Dismiss the prompt. */
     }
   else
@@ -1740,7 +1742,8 @@ verify_pin (app_t app, gpg_error_t (*pincb)(void*, const char *, char **),
           return err;
         }
 
-      err = iso7816_verify (app->slot, 0x81, pinvalue, strlen(pinvalue));
+      err = iso7816_verify (app_get_slot (app),
+                            0x81, pinvalue, strlen(pinvalue));
       xfree (pinvalue);
     }
   if (err)
@@ -1770,7 +1773,7 @@ verify_pin (app_t app, gpg_error_t (*pincb)(void*, const char *, char **),
    the ECDSA signature in X9.62 format (SEQ/INT(r)/INT(s))
 */
 static gpg_error_t
-do_sign (app_t app, const char *keyidstr, int hashalgo,
+do_sign (app_t app, ctrl_t ctrl, const char *keyidstr, int hashalgo,
          gpg_error_t (*pincb)(void*, const char *, char **),
          void *pincb_arg,
          const void *indata, size_t indatalen,
@@ -1807,6 +1810,8 @@ do_sign (app_t app, const char *keyidstr, int hashalgo,
   unsigned char algoid;
   int sw;
 
+  (void)ctrl;
+
   if (!keyidstr || !*keyidstr)
     return gpg_error (GPG_ERR_INV_VALUE);
 
@@ -1883,7 +1888,8 @@ do_sign (app_t app, const char *keyidstr, int hashalgo,
   if (err)
     return err;
 
-  sw = apdu_send_le (app->slot, 1, 0x80, 0x68, prkdf->key_reference, algoid,
+  sw = apdu_send_le (app_get_slot (app),
+                     1, 0x80, 0x68, prkdf->key_reference, algoid,
                      cdsblklen, cdsblk, 0, outdata, outdatalen);
   return iso7816_map_sw (sw);
 }
@@ -1897,7 +1903,7 @@ do_sign (app_t app, const char *keyidstr, int hashalgo,
    must match the criteria used for the attribute $AUTHKEYID.  See
    do_sign for calling conventions; there is no HASHALGO, though. */
 static gpg_error_t
-do_auth (app_t app, const char *keyidstr,
+do_auth (app_t app, ctrl_t ctrl, const char *keyidstr,
          gpg_error_t (*pincb)(void*, const char *, char **),
          void *pincb_arg,
          const void *indata, size_t indatalen,
@@ -1920,7 +1926,7 @@ do_auth (app_t app, const char *keyidstr,
     }
 
   algo = indatalen == 36? MD_USER_TLS_MD5SHA1 : GCRY_MD_SHA1;
-  return do_sign (app, keyidstr, algo, pincb, pincb_arg,
+  return do_sign (app, ctrl, keyidstr, algo, pincb, pincb_arg,
                   indata, indatalen, outdata, outdatalen);
 }
 
@@ -1969,7 +1975,7 @@ strip_PKCS15_padding(unsigned char *src, int srclen, unsigned char **dst,
 /* Decrypt a PKCS#1 V1.5 formatted cryptogram using the referenced
    key.  */
 static gpg_error_t
-do_decipher (app_t app, const char *keyidstr,
+do_decipher (app_t app, ctrl_t ctrl, const char *keyidstr,
              gpg_error_t (*pincb)(void*, const char *, char **),
              void *pincb_arg,
              const void *indata, size_t indatalen,
@@ -1984,6 +1990,8 @@ do_decipher (app_t app, const char *keyidstr,
   size_t p1blklen;
   int sw;
 
+  (void)ctrl;
+
   if (!keyidstr || !*keyidstr || !indatalen)
     return gpg_error (GPG_ERR_INV_VALUE);
 
@@ -2018,7 +2026,8 @@ do_decipher (app_t app, const char *keyidstr,
   if (err)
     return err;
 
-  sw = apdu_send_le (app->slot, 1, 0x80, 0x62, prkdf->key_reference, 0x21,
+  sw = apdu_send_le (app_get_slot (app),
+                     1, 0x80, 0x62, prkdf->key_reference, 0x21,
                      p1blklen, p1blk, 0, &rspdata, &rspdatalen);
   err = iso7816_map_sw (sw);
   if (err)
@@ -2044,13 +2053,13 @@ do_decipher (app_t app, const char *keyidstr,
 gpg_error_t
 app_select_sc_hsm (app_t app)
 {
-  int slot = app->slot;
+  int slot = app_get_slot (app);
   int rc;
 
   rc = iso7816_select_application (slot, sc_hsm_aid, sizeof sc_hsm_aid, 0);
   if (!rc)
     {
-      app->apptype = "SC-HSM";
+      app->apptype = APPTYPE_SC_HSM;
 
       app->app_local = xtrycalloc (1, sizeof *app->app_local);
       if (!app->app_local)
@@ -2064,6 +2073,8 @@ app_select_sc_hsm (app_t app)
         goto leave;
 
       app->fnc.deinit = do_deinit;
+      app->fnc.prep_reselect = NULL;
+      app->fnc.reselect = NULL;
       app->fnc.learn_status = do_learn_status;
       app->fnc.readcert = do_readcert;
       app->fnc.getattr = do_getattr;
